"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
function buildQueryString(queryStrings) {
    return Object.keys(queryStrings)
        .map(function (key) { return key + "=" + queryStrings[key]; })
        .join('&');
}
function getUrl(event) {
    var azEvent = event;
    return (event.path || (azEvent.req && (azEvent.req.url || azEvent.req.originalUrl)));
}
function buildFullUrl(event, queryStrings) {
    var url = getUrl(event);
    return !!queryStrings ? url + "?" + buildQueryString(queryStrings) : url;
}
function serverlessHapi(server, 
// Function to be called when `server.initialize` fails.
// According to the hapi documentation we should abort as soon as this
// function is called and contains an error. This is because the server
// is considered to be in an undefined state. It is recommended to assert
// that no error has been returned. Read more at hapi's documentation:
// https://github.com/hapijs/hapi/blob/v16/API.md#serverinitializecallback
onInitError, options) {
    if (options === void 0) { options = { filterHeaders: true, stringifyBody: true }; }
    return function (event, _context, callback) {
        var azEvent = event;
        var queryStrings = event.queryStringParameters;
        var hapiOptions = {
            method: event.httpMethod || (azEvent.req && azEvent.req.method),
            url: buildFullUrl(event, queryStrings),
            headers: event.headers || (azEvent.req && azEvent.req.headers),
            payload: event.body || (azEvent.req && azEvent.req.body),
        };
        server.initialize(function (error) {
            return error
                ? onInitError(error)
                : server.inject(hapiOptions, function (hapiResponse) {
                    var doneCallback = callback || azEvent.done;
                    var statusCode = hapiResponse.statusCode;
                    var body = hapiResponse.result;
                    // We need to remove hapi's content-encoding, transfer-encoding
                    // because lambda usually set's these.
                    // We filter away them by default but this can be changed
                    // by setting the `filterHeaders option to false.
                    var getHeaders = function () {
                        var _a = hapiResponse.headers, _ = _a["content-encoding"], __ = _a["transfer-encoding"], headers = __rest(_a, ['content-encoding', 'transfer-encoding']);
                        return options.filterHeaders ? headers : hapiResponse.headers;
                    };
                    var headers = getHeaders();
                    var cookieHeader = headers['set-cookie'];
                    // Lambda get's in trouble if we send back an array...
                    var setCookieHeader = Array.isArray(cookieHeader)
                        ? cookieHeader[0]
                        : cookieHeader;
                    var data = {
                        statusCode: statusCode,
                        body: options.stringifyBody
                            ? typeof body === 'string'
                                ? body
                                : JSON.stringify(body)
                            : body,
                        headers: __assign({}, headers, (_a = {}, _a['set-cookie'] = setCookieHeader, _a)),
                    };
                    var response = !callback ? { res: data } : data;
                    doneCallback(null, response);
                    var _a;
                });
        });
    };
}
exports.serverlessHapi = serverlessHapi;
